from django.core.paginator import Paginator, InvalidPage, EmptyPage
from django.shortcuts import render, redirect

from lib.access_auth import permission_needed
from portailuser.models import ApplicationLog, LogEntry
from gestion.logs.models import ReglageLog

ENTRIES_PER_PAGE = 20


@permission_needed("root")
def log_list(request, app_name=None, page=None):
    applications = ApplicationLog.objects.all()

    if request.method == "POST":
        app_name = request.POST.get("filter")

        if app_name and (ApplicationLog.objects.filter(name=app_name).exists() or app_name == "perso"):
            return redirect("gestion:logs:log_list", app_name=app_name)

        return redirect("gestion:logs:log_list")

    if app_name and not (ApplicationLog.objects.filter(name=app_name).exists() or app_name == "perso"):
        return redirect("gestion:logs:log_list")

    if app_name:
        if app_name == "perso":
            apps_name = [app.app for app in ReglageLog.objects.filter(user=request.user)]
            log_entries = LogEntry.objects.filter(application__in=apps_name)
        else:
            log_entries = LogEntry.objects.filter(application__name=app_name)
    else:
        log_entries = LogEntry.objects.filter()

    log_entries = log_entries.order_by("-date")
    proposition_nb_page = [20, 50, 100, 500]
    nb_log = int(request.GET.get("nb_log", ENTRIES_PER_PAGE))
    if nb_log not in proposition_nb_page:
        nb_log = ENTRIES_PER_PAGE

    paginator = Paginator(log_entries, nb_log)

    try:
        page = int(request.GET.get("page", "1"))
    except ValueError:
        page = 1

    try:
        page_obj = paginator.page(page)
    except (EmptyPage, InvalidPage):
        page_obj = paginator.page(paginator.num_pages)

    return render(request, "gestion/logs/log_list.html", locals())


@permission_needed("root")
def reglage(request):
    reglages_user = ReglageLog.objects.filter(user=request.user)

    apps = [(app, reglages_user.filter(app=app).exists()) for app in ApplicationLog.objects.all()]

    if request.method == "POST":
        for (app, b) in apps:
            if f"app_{app.pk}" in request.POST and not b:
                ReglageLog(user=request.user, app=app).save()
            if f"app_{app.pk}" not in request.POST and b:
                reglages_user.get(app=app).delete()

        return redirect("gestion:logs:log_list", app_name="perso")

    if not reglages_user:
        apps = [(app, True) for app in ApplicationLog.objects.all()]

    c = {
        "apps": apps,
    }

    return render(request, "gestion/logs/reglage.html", c)
